(function () {
  window.__frontend = {
    version: '4.0.3',
    deps: {
      '@labelu/audio-annotator-react': '1.3.1',
      '@labelu/components-react': '1.3.1',
      '@labelu/formatter': '1.0.2',
      '@labelu/image': '1.0.1',
      '@labelu/image-annotator-react': '1.0.2',
      '@labelu/interface': '1.3.0',
      '@labelu/video-annotator-react': '1.3.1',
      '@labelu/video-react': '1.2.4',
    },
  };
})();
